searchData={"items":[{"type":"module","title":"YOLO","doc":"This module provides the main entry point and library context for YOLO object detection in Elixir.\nIt delegates to the underlying model and utility modules for most functionality.","ref":"YOLO.html"},{"type":"function","title":"YOLO.detect/2","doc":"","ref":"YOLO.html#detect/2"},{"type":"function","title":"YOLO.detect/3","doc":"","ref":"YOLO.html#detect/3"},{"type":"function","title":"YOLO.load/1","doc":"","ref":"YOLO.html#load/1"},{"type":"function","title":"YOLO.to_detected_objects/2","doc":"","ref":"YOLO.html#to_detected_objects/2"},{"type":"behaviour","title":"YOLO.FrameScaler","doc":"Behaviour module defining callbacks for part of the image preprocessing required by YOLO models.\n\nThe callbacks in this module handle three key preprocessing steps:\n\n1. Getting the input image dimensions to calculate proper scaling\n2. Resizing the image while preserving aspect ratio\n3. Converting the image to an Nx tensor in the correct format\n\nThe `YOLO.FrameScalers.fit/3` function uses the callbacks to:\n- Scale the image to fit the target dimensions while maintaining aspect ratio\n- Pad any remaining space with gray color (value 114).\n  [Why use 114,114,114?](https://github.com/ultralytics/ultralytics/issues/14584)\n\nImplementations of this behaviour provide the interface between different image\nprocessing libraries and the common YOLO preprocessing requirements.","ref":"YOLO.FrameScaler.html"},{"type":"callback","title":"YOLO.FrameScaler.image_resize/2","doc":"Resizes the input image according to the scaling configuration.\n\nThe scaling configuration contains the target dimensions and scaling factors needed to resize\nthe image while preserving aspect ratio. The resize is done in the implementation's native\nformat before later conversion to an Nx tensor via `image_to_nx/1`.","ref":"YOLO.FrameScaler.html#c:image_resize/2"},{"type":"callback","title":"Arguments - YOLO.FrameScaler.image_resize/2","doc":"* `image` - Input image in implementation's native format\n  * `scaling_config` - `ScalingConfig` struct containing target dimensions and scale factors","ref":"YOLO.FrameScaler.html#c:image_resize/2-arguments"},{"type":"callback","title":"Returns - YOLO.FrameScaler.image_resize/2","doc":"* Resized image in implementation's native format","ref":"YOLO.FrameScaler.html#c:image_resize/2-returns"},{"type":"callback","title":"YOLO.FrameScaler.image_shape/1","doc":"Returns the shape of the input image as a tuple of `{width, height, channels}`.\nThe shape information is used to properly scale and pad images while maintaining aspect ratio.","ref":"YOLO.FrameScaler.html#c:image_shape/1"},{"type":"callback","title":"YOLO.FrameScaler.image_to_nx/1","doc":"Converts the implementation's native image format to an Nx tensor.\nThe tensor should have shape `{height, width, channels}` where channels is typically 3 for RGB images.","ref":"YOLO.FrameScaler.html#c:image_to_nx/1"},{"type":"module","title":"YOLO.FrameScalers","doc":"Provides functions for resizing images while preserving aspect ratio and handling\npadding to reach target dimensions.","ref":"YOLO.FrameScalers.html"},{"type":"module","title":"Available scalers - YOLO.FrameScalers","doc":"The following scaler implementations are currently provided:\n\n* `YOLO.FrameScalers.EvisionScaler` - For scaling images using `Evision`\n* `YOLO.FrameScalers.ImageScaler` - For scaling images using `Image` library\n* `YOLO.FrameScalers.NxIdentityScaler` - For when input is already an appropriately sized `Nx` tensor\n\n```elixir\n# For Evision Mat images:\nYOLO.detect(model, mat, frame_scaler: YOLO.FrameScalers.EvisionScaler)\n\n# For Image library images:\nYOLO.detect(model, image, frame_scaler: YOLO.FrameScalers.ImageScaler)\n\n# For pre-sized Nx tensors:\nYOLO.detect(model, tensor, frame_scaler: YOLO.FrameScalers.NxIdentityScaler)\n```","ref":"YOLO.FrameScalers.html#module-available-scalers"},{"type":"function","title":"YOLO.FrameScalers.fit/3","doc":"Scales and pads an image to fit target dimensions.\nIt's used in the model `preprocess/3` callback.","ref":"YOLO.FrameScalers.html#fit/3"},{"type":"function","title":"Arguments - YOLO.FrameScalers.fit/3","doc":"* `image` - Input image in implementation's native format\n  * `target_shape` - Target dimensions as `{width, height}` tuple\n  * `resizer_module` - Module implementing the FrameScaler behaviour","ref":"YOLO.FrameScalers.html#fit/3-arguments"},{"type":"function","title":"Returns - YOLO.FrameScalers.fit/3","doc":"* `{image_tensor, scaling_config}` where image_tensor is the processed Nx tensor\n    and scaling_config is a struct containing scaling and padding information\n\nThe returned `image_tensor` has shape `{height, width, channels}`.","ref":"YOLO.FrameScalers.html#fit/3-returns"},{"type":"function","title":"YOLO.FrameScalers.scale_bboxes_to_original/2","doc":"Adjusts bounding box coordinates and sizes from the model input dimensions back to the original image dimensions.\n`bboxes` is a list of rows, where rows are lists of 6 elements `[cx, cy, w, h, prob, class_idx]`.\n\nReturns a list of scaled bboxes.\n\n# Example\nFor a YOLOv8n model processing an image resized to 640x640, use the following to scale\nthe bounding boxes back to the original 1920x1080 image:\n\n```\nscale_bboxes_to_original(bboxes, scaling_config)\n```","ref":"YOLO.FrameScalers.html#scale_bboxes_to_original/2"},{"type":"module","title":"YOLO.FrameScalers.EvisionScaler","doc":"Implementation of the `YOLO.FrameScaler` behaviour for `Evision.Mat` objects (`:evision` library).","ref":"YOLO.FrameScalers.EvisionScaler.html"},{"type":"module","title":"YOLO.FrameScalers.ImageScaler","doc":"Implementation of the YOLO.FrameScaler behaviour for Vix.Vips.Image objects (`:image` library).","ref":"YOLO.FrameScalers.ImageScaler.html"},{"type":"module","title":"YOLO.FrameScalers.NxIdentityScaler","doc":"`FrameScaler` for when the input image is already a tensor with shape {height, width, 3}\nand no scaling is needed because width and height already match the model input size.\nThis is useful when calling `YOLO.detect/3` with a tensor that already matches the\nexpected dimensions.","ref":"YOLO.FrameScalers.NxIdentityScaler.html"},{"type":"module","title":"YOLO.FrameScalers.ScalingConfig","doc":"Stores image scaling and padding configuration used during YOLO model preprocessing.","ref":"YOLO.FrameScalers.ScalingConfig.html"},{"type":"type","title":"YOLO.FrameScalers.ScalingConfig.t/0","doc":"","ref":"YOLO.FrameScalers.ScalingConfig.html#t:t/0"},{"type":"behaviour","title":"YOLO.Model","doc":"Defines a behaviour for implementing YOLO object detection models.\n\nThis module provides the structure for loading and running YOLO models for object detection.\nThe default implementation is `YOLO.Models.YoloV8`, but you can create custom implementations\nfor other YOLO variants.","ref":"YOLO.Model.html"},{"type":"behaviour","title":"Required Callbacks - YOLO.Model","doc":"To implement this behaviour, you need to define these functions:\n\n- `preprocess/3`: Prepares an input image for the model\n  - Takes a model struct, input image, and options\n  - Returns `{preprocessed_tensor, scaling_config}`\n  - See `YOLO.Models.YoloV8` for an example implementation\n\n- `postprocess/4`: Processes the model's raw output into detected objects\n  - Takes model struct, model output tensor, scaling config, and options\n  - Returns list of detected objects as `[cx, cy, w, h, prob, class_idx]`\n  - Handles tasks like non-maximum suppression and coordinate scaling","ref":"YOLO.Model.html#module-required-callbacks"},{"type":"behaviour","title":"Optional Callbacks - YOLO.Model","doc":"- `init/1`: Initializes the model. This is called when the model is loaded. It's the place where you can, for example, generate and store in `model_data` things like grids and expanded strides that will be used later in other callbacks.","ref":"YOLO.Model.html#module-optional-callbacks"},{"type":"behaviour","title":"Types - YOLO.Model","doc":"- `t()`: The model struct containing:\n  - `:ref` - Reference to loaded ONNX model\n  - `:model_impl` - Module implementing this behaviour\n  - `:shapes` - Input/output tensor shapes\n  - `:classes` - Map of class indices to labels\n  - `:model_data` - Model-specific data, e.g. grids and expanded strides for YOLOX\n\n- `detected_object()`: Map containing detection results:\n  - `:bbox` - Bounding box coordinates (cx, cy, w, h)\n  - `:class` - Detected class name\n  - `:class_idx` - Class index\n  - `:prob` - Detection probability","ref":"YOLO.Model.html#module-types"},{"type":"callback","title":"YOLO.Model.init/2","doc":"Initialize the model. This is called when the model is loaded. It's the place where you can, for example, generate and store in `model_data` things like grids and expanded strides that will be used later in other callbacks.","ref":"YOLO.Model.html#c:init/2"},{"type":"callback","title":"Parameters - YOLO.Model.init/2","doc":"* `model`\n  * `options` : options passed to YOLO.load/2","ref":"YOLO.Model.html#c:init/2-parameters"},{"type":"callback","title":"Returns - YOLO.Model.init/2","doc":"* The updated `YOLO.Model` struct","ref":"YOLO.Model.html#c:init/2-returns"},{"type":"callback","title":"YOLO.Model.postprocess/4","doc":"Post-processes the model's raw output to produce a list of detected objects.\n\nThe raw output from the model is a tensor containing bounding box coordinates and class probabilities\nfor each candidate detection.\n\nFor example, YOLOv8 outputs a `{1, 84, 8400}` tensor where:\n  - 84 represents 4 bbox coordinates + 80 class probabilities\n  - 8400 represents the number of candidate detections\n\nReturns a list of detections where each detection is a list of 6 elements:\n```elixir\n[cx, cy, w, h, prob, class_idx]\n```\nwhere:\n- `cx`, `cy`: center x,y coordinates of bounding box\n- `w`, `h`: width and height of bounding box\n- `prob`: detection probability\n- `class_idx`: class index\n\nThe implementation should:\n1. Filter low probability detections\n2. Apply non-maximum suppression (NMS) to remove overlapping boxes\n3. Scale back the coordinates using the `scaling_config` and `YOLO.FrameScaler` module, since\n   the detections are based on the model's input resolution rather than the original image size\n\nSee `YOLO.Models.YoloV8.postprocess/4` for a reference implementation.","ref":"YOLO.Model.html#c:postprocess/4"},{"type":"callback","title":"YOLO.Model.preprocess/3","doc":"Prepares input image tensors for the model.","ref":"YOLO.Model.html#c:preprocess/3"},{"type":"callback","title":"Parameters - YOLO.Model.preprocess/3","doc":"* `model` - The YOLO.Model struct containing model information\n  * `image` - Input image in implementation's native format (e.g. Evision.Mat)\n  * `options` - Keyword list of options:\n    * `:frame_scaler` - Module implementing YOLO.FrameScaler behaviour (required)","ref":"YOLO.Model.html#c:preprocess/3-parameters"},{"type":"callback","title":"Returns - YOLO.Model.preprocess/3","doc":"* `{input_tensor, scaling_config}` tuple where:\n    * `input_tensor` is the preprocessed Nx tensor ready for model input, where shape is `{1, channels, height, width}`\n    * `scaling_config` contains scaling/padding info for postprocessing\n\nLook at the `YOLO.Models.YoloV8.preprocess/3` implementation to see how this callback is implemented.","ref":"YOLO.Model.html#c:preprocess/3-returns"},{"type":"type","title":"YOLO.Model.classes/0","doc":"","ref":"YOLO.Model.html#t:classes/0"},{"type":"type","title":"YOLO.Model.detected_object/0","doc":"","ref":"YOLO.Model.html#t:detected_object/0"},{"type":"type","title":"YOLO.Model.shape/0","doc":"","ref":"YOLO.Model.html#t:shape/0"},{"type":"type","title":"YOLO.Model.t/0","doc":"","ref":"YOLO.Model.html#t:t/0"},{"type":"module","title":"YOLO.Models","doc":"This module handles loading YOLO models and running object detection on images.\nThe `YOLO.Model` behaviour can be implemented for various YOLO variants.\nThe supported models are:\n- `YOLO.Models.Ultralytics`: Implements models from the Ultralytics YOLO family (https://www.ultralytics.com).\n- `YOLO.Models.YOLOX`: Implements the YOLOX object detection model (https://github.com/Megvii-BaseDetection/YOLOX).","ref":"YOLO.Models.html"},{"type":"module","title":"Main Functions - YOLO.Models","doc":"The key functions you'll use are:\n\n- `YOLO.Models.load/1`: Loads a YOLO model with required options\n  ```elixir\n  YOLO.Models.load(model_path: \"path/to/model.onnx\",\n                  classes_path: \"path/to/classes.json\",\n                  model_impl: YOLO.Models.Ultralytics)\n  ```\n\n- `YOLO.Models.detect/3`: Runs object detection on an image\n  ```elixir\n  YOLO.Models.detect(model, image, prob_threshold: 0.5)\n  ```","ref":"YOLO.Models.html#module-main-functions"},{"type":"function","title":"YOLO.Models.detect/3","doc":"Performs object detection on an image using a loaded YOLO model.","ref":"YOLO.Models.html#detect/3"},{"type":"function","title":"Arguments - YOLO.Models.detect/3","doc":"* `model` - A loaded `YOLO.Model.t()` struct\n* `image` - Input image in the format expected by the frame scaler (e.g. `Evision.Mat`)\n* `opts` - Detection options","ref":"YOLO.Models.html#detect/3-arguments"},{"type":"function","title":"Options - YOLO.Models.detect/3","doc":"* `prob_threshold` - Minimum probability threshold for detections (default: `0.25`)\n* `iou_threshold` - IoU threshold for non-maximum suppression (default: `0.45`)\n* `frame_scaler` - Module implementing `YOLO.FrameScaler` behaviour (default: `YOLO.FrameScalers.EvisionScaler`)","ref":"YOLO.Models.html#detect/3-options"},{"type":"function","title":"Returns - YOLO.Models.detect/3","doc":"A list of detections, where each detection is a list `[cx, cy, w, h, prob, class_idx]`:\n* `cx`, `cy` - Center coordinates of bounding box\n* `w`, `h` - Width and height of bounding box\n* `prob` - Detection probability\n* `class_idx` - Class index\n\nThe output can be converted to structured maps using `to_detected_objects/1`.","ref":"YOLO.Models.html#detect/3-returns"},{"type":"function","title":"Example - YOLO.Models.detect/3","doc":"model\n    |> YOLO.Model.detect(image, prob_threshold: 0.5)\n    |> YOLO.Model.to_detected_objects()","ref":"YOLO.Models.html#detect/3-example"},{"type":"function","title":"YOLO.Models.load/1","doc":"Loads a YOLO model from an ONNX file.","ref":"YOLO.Models.html#load/1"},{"type":"function","title":"Required Options - YOLO.Models.load/1","doc":"* `model_path` - Path to the `.onnx` model file","ref":"YOLO.Models.html#load/1-required-options"},{"type":"function","title":"Optional Options - YOLO.Models.load/1","doc":"* `model_impl` - Module implementing the `YOLO.Model` behaviour (default: YOLO.Models.Ultralytics)\n* `classes_path` - Path to the `.json` file containing class labels. If not provided, classes will not be loaded.\n\n* `eps` - List of execution providers to pass to Ortex (e.g. `[:coreml]`, `[:cuda]`, `[:tensorrt]`, `[:directml]`), default: `[:cpu]`\n* `json_decoder` - Function to decode JSON strings (default: `&:json.decode/1`)","ref":"YOLO.Models.html#load/1-optional-options"},{"type":"function","title":"Returns - YOLO.Models.load/1","doc":"A `YOLO.Model.t()` struct containing:\n* `ref` - Reference to the loaded ONNX model\n* `model_impl` - The module implementing the model version\n* `classes` - Map of class indices to labels\n* `shapes` - Input/output tensor shapes\n* `model_data` - Model-specific data","ref":"YOLO.Models.html#load/1-returns"},{"type":"function","title":"Examples - YOLO.Models.load/1","doc":"```elixir\n  yolox_model = YOLO.Model.load(\n    model_path: \"models/yolox_s.onnx\",\n    classes_path: \"models/coco_classes.json\",\n    model_impl: YOLO.Models.YOLOX\n  )\n  ```\n\n\n  ```elixir\n  ultralytics_model = YOLO.Model.load(\n    model_path: \"models/yolo11n.onnx\",\n    classes_path: \"models/coco_classes.json\",\n    model_impl: YOLO.Models.Ultralytics\n  )\n  ```","ref":"YOLO.Models.html#load/1-examples"},{"type":"function","title":"YOLO.Models.load_classes/3","doc":"Loads class labels from a JSON file and adds them to an existing model.","ref":"YOLO.Models.html#load_classes/3"},{"type":"function","title":"Arguments - YOLO.Models.load_classes/3","doc":"* `model` - A `YOLO.Model.t()` struct to update with class labels\n* `classes_path` - Path to the JSON file containing class labels\n* `options` - Keyword list of options","ref":"YOLO.Models.html#load_classes/3-arguments"},{"type":"function","title":"Options - YOLO.Models.load_classes/3","doc":"* `json_decoder` - Function to decode JSON strings (default: `&:json.decode/1`)","ref":"YOLO.Models.html#load_classes/3-options"},{"type":"function","title":"Returns - YOLO.Models.load_classes/3","doc":"An updated `YOLO.Model.t()` struct with the `classes` field populated.","ref":"YOLO.Models.html#load_classes/3-returns"},{"type":"function","title":"YOLO.Models.run/2","doc":"Runs inference on a preprocessed input tensor.","ref":"YOLO.Models.html#run/2"},{"type":"function","title":"Arguments - YOLO.Models.run/2","doc":"* `model` - A loaded `YOLO.Model.t()` struct\n* `image_tensor` - Preprocessed input tensor matching model's expected shape","ref":"YOLO.Models.html#run/2-arguments"},{"type":"function","title":"Returns - YOLO.Models.run/2","doc":"The raw output tensor from the model. For YOLOv8n:\n* Input shape: `{1, 3, 640, 640}`\n* Output shape: `{1, 84, 8400}`\n\nThis is typically used internally by `detect/3` and shouldn't need to be called directly.","ref":"YOLO.Models.html#run/2-returns"},{"type":"module","title":"YOLO.Models.Ultralytics","doc":"Ultralytics model implementation for preprocessing input images\nand postprocessing detections using non-maximum suppression (NMS).\n\nSupports YOLOv8 and YOLOv11 models trained on the COCO dataset (80 classes).","ref":"YOLO.Models.Ultralytics.html"},{"type":"function","title":"YOLO.Models.Ultralytics.postprocess/4","doc":"Post-processes the model's raw output to produce a filtered list of detected objects.\n\nThe raw output tensor has shape {1, 84, 8400} where:\n- First dimension (1) is the batch size\n- Second dimension (84) contains 4 bbox coordinates + 80 class probabilities per detection\n- Third dimension (8400) is the number of candidate detections\n\nThe processing steps are:\n1. Reshapes and transposes output to {8400, 84} format\n2. Applies non-maximum suppression (NMS) to filter overlapping detections\n3. Scales bounding boxes back to original image dimensions","ref":"YOLO.Models.Ultralytics.html#postprocess/4"},{"type":"function","title":"Arguments - YOLO.Models.Ultralytics.postprocess/4","doc":"* `model` - YOLO.Model struct containing model metadata\n  * `model_output` - Raw output tensor from model inference `{1, 84, 8400}`\n  * `scaling_config` - Scaling configuration from preprocessing step\n  * `opts` - Keyword list of options:\n    * `prob_threshold` - Minimum probability threshold for detections\n    * `iou_threshold` - IoU threshold for non-maximum suppression\n    * `nms_fun` - Optional custom NMS function (defaults to `YOLO.NMS.run/2`)","ref":"YOLO.Models.Ultralytics.html#postprocess/4-arguments"},{"type":"function","title":"Returns - YOLO.Models.Ultralytics.postprocess/4","doc":"List of detections where each detection is a list [cx, cy, w, h, prob, class_idx]:\n  * cx, cy - Center coordinates of bounding box\n  * w, h - Width and height of bounding box\n  * prob - Detection probability\n  * class_idx - Class index (0-79 corresponding to COCO classes)","ref":"YOLO.Models.Ultralytics.html#postprocess/4-returns"},{"type":"function","title":"YOLO.Models.Ultralytics.preprocess/3","doc":"Preprocesses an input image to match the model's required format.\n\nThe preprocessing steps are:\n1. Scales and pads the image to match model input dimensions while preserving aspect ratio\n2. Converts RGB to BGR color format (Ultralytics models expect BGR input)\n3. Normalizes pixel values to [0,1] range by dividing by 255\n4. Transposes dimensions to match model's expected format\n5. Adds batch dimension","ref":"YOLO.Models.Ultralytics.html#preprocess/3"},{"type":"function","title":"Arguments - YOLO.Models.Ultralytics.preprocess/3","doc":"* `model` - YOLO.Model struct containing model metadata\n  * `image` - Input image in implementation's native format\n  * `options` - Keyword list of options including `:frame_scaler` module","ref":"YOLO.Models.Ultralytics.html#preprocess/3-arguments"},{"type":"function","title":"Returns - YOLO.Models.Ultralytics.preprocess/3","doc":"* `{input_tensor, scaling_config}` tuple where:\n    * `input_tensor` has shape {1, 3, height, width}\n    * `scaling_config` contains scaling/padding info for postprocessing","ref":"YOLO.Models.Ultralytics.html#preprocess/3-returns"},{"type":"module","title":"YOLO.Models.YOLOX","doc":"YOLOX model implementation for preprocessing input images and postprocessing detections using non-maximum suppression (NMS).\n\nSupports YOLOX models found at [https://github.com/Megvii-BaseDetection/YOLOX](https://github.com/Megvii-BaseDetection/YOLOX)\n\nIf using a YOLOX model that was exported with `--decode_in_inference`, you can set\n`decode_head: false` in the YOLO.detect/3 options.\n\nYOLOX-Tiny and YOLOX-Nano models use 416x416, while other models use 640x640.","ref":"YOLO.Models.YOLOX.html"},{"type":"function","title":"YOLO.Models.YOLOX.calculate_max_prob_score_per_prediction/1","doc":"Calculates the detection score for each prediction as the product of the maximum class\nprobability and the objectness score.\n\nAdaptation of YOLO.NMS.filter_predictions/2, but calculates the correct score based on\nthe product of the maximum class probability and the objectness score which differs from Ultralytics\n\nRemoves prob_threshold filtering so that we can use Nx.Defn compilation for performance.","ref":"YOLO.Models.YOLOX.html#calculate_max_prob_score_per_prediction/1"},{"type":"function","title":"YOLO.Models.YOLOX.default_nms/3","doc":"","ref":"YOLO.Models.YOLOX.html#default_nms/3"},{"type":"function","title":"YOLO.Models.YOLOX.generate_grids_and_expanded_strides/2","doc":"","ref":"YOLO.Models.YOLOX.html#generate_grids_and_expanded_strides/2"},{"type":"function","title":"YOLO.Models.YOLOX.postprocess/4","doc":"Post-processes the model's raw output to produce a filtered list of detected objects.\n\nOptions:\n* `decode_head` - If true, decode the output head to map predictions to input image space.\n  Defaults to `true`. Can be set to `false` if using a YOLOX model that was exported with `--decode_in_inference`\n* `nms_fun` - Optional custom NMS function. Must calculate detection scores as the product of the maximum class\n  probability and the objectness score.\n* `prob_threshold` - Minimum probability threshold for detections\n* `iou_threshold` - IoU threshold for non-maximum suppression","ref":"YOLO.Models.YOLOX.html#postprocess/4"},{"type":"function","title":"YOLO.Models.YOLOX.preprocess/3","doc":"YOLOX input doesn't need to be normalized, so we resize and convert the image to a `{batch_size, channels, height, width}` tensor.\n\nTiny and Nano models use 416x416, while other models use 640x640.","ref":"YOLO.Models.YOLOX.html#preprocess/3"},{"type":"function","title":"YOLO.Models.YOLOX.process_bboxes/3","doc":"","ref":"YOLO.Models.YOLOX.html#process_bboxes/3"},{"type":"module","title":"YOLO.NMS","doc":"Elixir NMS (Non-Maximum Suppression)\n\nLearn more about Non-Maximum Suppression (NMS) at:\nhttps://builtin.com/machine-learning/non-maximum-suppression\n\nThis implementation applies NMS independently for each output class.\nThe following steps are executed for each class:\n1. Filter out all bounding boxes with maximum class probability below `prob_threshold` (default: `0.5`).\n2. Select the bounding box with the highest `prob`.\n3. Remove any remaining bounding boxes with an IoU >= `iou_threshold` (default: `0.5`).","ref":"YOLO.NMS.html"},{"type":"function","title":"YOLO.NMS.filter_predictions/3","doc":"Filters detections based on a confidence probability threshold.\n\nSelects detections from `model_output` where the highest class confidence score exceeds `prob_threshold`.","ref":"YOLO.NMS.html#filter_predictions/3"},{"type":"function","title":"Arguments - YOLO.NMS.filter_predictions/3","doc":"* `model_output`: Input tensor. The standard expected shape is `{detections, bbox+classes}`\n    (e.g., `{8400, 84}`). The function can also handle inputs with a leading\n    batch dimension (e.g., `{1, 8400, 84}`), effectively squeezing internally.\n    If the input shape is `{bbox+classes, detections}` (e.g., `{84, 8400}`) or\n    `{1, bbox+classes, detections}` (e.g., `{1, 84, 8400}`), set\n    `transpose?: true` for internal transposition. The first 4 elements\n    of the `bbox+classes` dimension must be the bounding box coordinates.\n\n  * `prob_threshold`: Minimum confidence probability to keep a detection.\n\n  * `opts`: Keyword list options:\n    - `:transpose?` (boolean, default: `false`): If `true`, transpose the\n      input `model_output` before processing.","ref":"YOLO.NMS.html#filter_predictions/3-arguments"},{"type":"function","title":"Returns - YOLO.NMS.filter_predictions/3","doc":"A list of detections `[cx, cy, w, h, prob, class_idx]`, sorted descending\nby `prob`. Returns `[]` if no detections meet the threshold.","ref":"YOLO.NMS.html#filter_predictions/3-returns"},{"type":"function","title":"YOLO.NMS.iou/2","doc":"IoU (Intersection over Union) measures the ratio of the area of overlap between\ntwo bounding boxes to the area of their union.\n\nIoU = A ∩ B / A U B","ref":"YOLO.NMS.html#iou/2"},{"type":"function","title":"YOLO.NMS.nms/2","doc":"Applies Non-Maximum Suppression (NMS) for each class, discarding bounding boxes with an\nIoU exceeding the specified `iou_threshold`.","ref":"YOLO.NMS.html#nms/2"},{"type":"function","title":"YOLO.NMS.run/2","doc":"Runs both `filter_predictions/2` and `nms/2`.\n\n1. Filters out detections with a probability below `prob_threshold` (`p < prob_threshold`)\n2. Applies Non-Maximum Suppression (NMS) for each class, discarding bounding boxes with an\n   IoU exceeding the specified `iou_threshold`.\n\nThe input `tensor` is the YOLO model output that can have different shapes depending on the model.\n\nWhen `transpose: false` (default):\n- Shape `{1=batch_size, num_detections, bbox_coords + num_classes}` or `{num_detections, bbox_coords + num_classes}`\n- Example: `{1, 8400, 84}` or `{8400, 84}` (where 4 is the number of bounding box coordinates and 80 is the number of classes)\n\nWhen `transpose: true`:\n- Shape `{bbox_coords + num_classes, num_detections}` or `{1, bbox_coords + num_classes, num_detections}`\n- Example: `{84, 8400}` or `{1, 84, 8400}` (where 4 is the number of bounding box coordinates and 80 is the number of classes)\n\nWhere:\n- `num_detections`: Number of candidate detections (varies by model)\n- `bbox_coords`: Number of bounding box coordinates (4 for Ultralytics YOLO)\n- `num_classes`: Number of classes (80 for Ultralytics YOLO trained on COCO dataset)\n\nReturns a list of `[bbox_cx, bbox_cy, bbox_w, bbox_h, prob, class_idx]`.","ref":"YOLO.NMS.html#run/2"},{"type":"module","title":"YOLO.Utils","doc":"Utils to transform the model input and output.","ref":"YOLO.Utils.html"},{"type":"function","title":"YOLO.Utils.to_detected_objects/2","doc":"Maps `[cx, cy, w, h, prob, class_idx]` rows to `YOLO.Model.detected_object()`\n\n# Example\n```\niex> YOLO.Utils.to_detected_objects([ [100, 200, 20, 30, 0.7, 2] ], yolov8n_model.classes)\n[%{\n  bbox: %{cx: 100, cy: 200, w: 20, h: 30},\n  prob: 0.7,\n  class_idx: 2,\n  class: \"car\"\n}]\n```","ref":"YOLO.Utils.html#to_detected_objects/2"},{"type":"extras","title":"README","doc":"[Getting Started](#getting-started) | [LiveBook Examples](#livebook-examples) | [Benchmarks](guides/benchmarks.md) | [Under the Hood](guides/under_the_hood.md)\n\nYou Only Look Once (YOLO) is a cutting-edge, real-time object detection system. It works by predicting bounding boxes and class probabilities directly from an input image in a single evaluation, making it exceptionally fast compared to other object detection methods. YOLO models are widely used in applications like autonomous driving, surveillance, and robotics due to their balance of speed and accuracy.  \n\n![traffic image](guides/images/traffic_detections.jpg)\n\n\n\n`YOLO` is an Elixir library designed to simplify object detection by providing seamless integration of YOLO models. With this library, you can efficiently utilize the power of YOLO for real-time object detection.  \n\nThe library is built with the following objectives:  \n\n1. **Speed**  \n   Achieve real-time object detection performance, even on resource-constrained devices like a MacBook Air M3. Details about performance can be found in the [benchmarks section](#benchmarks).  \n\n2. **Ease of Use**  \n   Designed to be straightforward and user-friendly, the library enables developers to load and detect objects with just two function calls.\n\n3. **Extensibility**  \n   Supporting Ultralytics YOLO and YOLOX models (in various sizes), the library is built around a `YOLO.Model` behavior. It leverages ONNX and Ortex and the library is not limited to specific model architectures - it can handle any number of detection candidates and classes, making it perfect for custom-trained models.","ref":"readme.html"},{"type":"extras","title":"Supported Models - README","doc":"The library requires YOLO models in ONNX format, which combines both the neural network architecture and pre-trained weights. Currently, we support two model families:\n\n* **YOLOX**: Pre-converted ONNX models are available directly from the [YOLOX repository](https://github.com/Megvii-BaseDetection/YOLOX/tree/main/demo/ONNXRuntime)\n* [**Ultralytics**](guides/ultralytics_to_onnx.md): These models need to be converted from PyTorch (.pt) to ONNX format (https://github.com/ultralytics/assets/releases/tag/v8.3.0) [follow this simple guide to prepare the model in ONNX format](guides/ultralytics_to_onnx.md)\n\n\nAll these models come in different sizes, like *n (nano)*, *s (small)*, *m (medium)*, *l (large)*, and *x (extra-large)*. Larger models offer better performance in terms of classification accuracy and object detection but require more computational resources and memory.","ref":"readme.html#supported-models"},{"type":"extras","title":"Getting Started - README","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Installation and Configuration - README","doc":"```elixir\n# mix.exs\n\ndefp deps do\n  {:yolo, \">= 0.2.0\"},\n\n  # I'm using EXLA as Nx backend\n  # Nx is mostly used for pre/post processing\n  {:exla, \"~> 0.9.2\"},\n  # evision for image processing (you can use :image instead)\n  {:evision, \"~> 0.2.0\"}\nend\n```\n\nTo enable hardware acceleration in Ortex based on platform (CoreML on Mac, DirectML on Windows, CUDA/TensorRT on Linux)\n```elixir\n# Mac with CoreML\nconfig :ortex, Ortex.Native, features: [:coreml]\n\n# Windows machine, DirectML\nconfig :ortex, Ortex.Native, features: [:directml]\n\n#CUDA/TensorRT\nconfig :ortex, Ortex.Native, features: [:cuda, :tensorrt]\n```\n\nIn case you run YOLO on [LiveBook](https://livebook.dev/) or iex\n\n```elixir\nMix.install([\n  :exla,\n  # evision for image processing (you can use :image instead)\n  :evision,\n\n  {:yolo, \">= 0.2.0\"}\n])\n```\nAnd in case you want, for example, CUDA acceleration\n\n```elixir\nMix.install([\n  :exla,\n  # evision for image processing (you can use :image instead)\n  :evision,\n\n  {:yolo, \">= 0.2.0\"}\n],\n[\n  config: [\n    # an example where we use CUDA acceleration for both Ortex and EXLA\n    ortex: {Ortex.Native, features: [:cuda]},\n    nx: [default_backend: {EXLA.Backend, client: :cuda}]\n  ]\n])\n```","ref":"readme.html#installation-and-configuration"},{"type":"extras","title":"Load and Run - README","doc":"Here we are using YOLOX, for this example, but you can use ultralytics just by loading the ultralytics model onnx and setting `model_impl` to `YOLO.Models.Ultralytics` (which is the default `model_impl` implementation).\n\nPre-converted YOLOX ONNX models are available directly on the [YOLOX repository](https://github.com/Megvii-BaseDetection/YOLOX/tree/main/demo/ONNXRuntime).\n\nLoading YOLOX nano, in this case with `:cpu` Execution provider.\n\n```elixir\nmodel = YOLO.load(\n  model_impl: YOLO.Models.YOLOX,\n  model_path: \"models/yolox_nano.onnx\", \n  classes_path: \"models/coco_classes.json\", \n\n  # Ortex execution providers (same as the `:ortex` config)\n  # Mac with CoreML\n  eps: [:cpu] # or [:coreml], [:directml] or [:cuda, :tensorrt], default: [:cpu]\n)\n```\n\nIf instead you want to load an Ultralytics YOLO model \n```elixir\nmodel = YOLO.load(\n  model_impl: YOLO.Models.Ultralytics,\n  model_path: \"models/yolo11n.onnx\",\n  ...\n)\n```\n\nLoading the traffic image and running YOLO.\n```elixir\nmat = Evision.imread(\"guides/images/traffic.jpg\")\n\nmodel\n|> YOLO.detect(mat)\n|> YOLO.to_detected_objects(model.classes)    \n\n```\n\nThis code returns a list of detected objects along with their bounding box coordinates.\n\n```elixir\n[\n  %{\n    class: \"person\",\n    prob: 0.57\n    bbox: %{h: 126, w: 70, cx: 700, cy: 570},\n    class_idx: 0\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.61\n    bbox: %{h: 102, w: 71, cx: 726, cy: 738},\n    class_idx: 1\n  },\n  %{class: \"car\", prob: 0.62, bbox: %{h: 87, w: 102, cx: 1039, cy: 268}, class_idx: 2},\n  ...\n]\n```","ref":"readme.html#load-and-run"},{"type":"extras","title":"Visualize detection results - README","doc":"For visualizing detection results on images, you can use the [KinoYOLO](https://github.com/poeticoding/kino_yolo) utility library. While the name suggests Kino integration, it only depends on the `image` package. The library is currently available on GitHub's main branch (not yet published to Hex).\n\n```elixir\n# using Image instead of Evision\nimage = Image.open(\"guides/images/traffic.jpg\")\n\ndetected_objects = \n  model\n  # here we are passing the image and we need the YOLO.FrameScalers.ImageScaler\n  # instead of the default `YOLO.FrameScalers.EvisionScaler`\n  |> YOLO.detect(image, frame_scaler: YOLO.FrameScalers.ImageScaler)\n  |> YOLO.to_detected_objects(model.classes)    \n\nKinoYOLO.Draw.draw_detected_objects(image, detected_objects)\n```\n\n![traffic image](guides/images/traffic_detections.jpg)\n\nAlternatively, you can process the image with `Evision` through the YOLO pipeline, convert the resulting `mat` to an `Image` type, and visualize the detections using `KinoYOLO.Draw.draw_detected_objects/2`.","ref":"readme.html#visualize-detection-results"},{"type":"extras","title":"Post-processing - Make it faster with FastNMS - README","doc":"The default `YOLO.NMS` implementation (from version 0.2) uses `Nx.Defn` for postprocessing and is already quite fast for real-time YOLO detection. However, if you need to squeeze every millisecond of performance, you can use the [`YoloFastNMS`](https://github.com/poeticoding/yolo_fast_nms) library which provides a Rust NIF implementation that should be 2x faster.\n\n```elixir\ndef deps do\n  [\n    {:yolo_fast_nms, \"~> 0.2\"}\n  ]\nend\n```\n\nThen you can run the detection with FastNMS by passing the `YoloFastNMS.run/2` function to `detect/3`.\n```elixir\nYOLO.detect(model, mat, nms_fun: &YoloFastNMS.run/2)\n```","ref":"readme.html#post-processing-make-it-faster-with-fastnms"},{"type":"extras","title":"Models trained on different datasets - README","doc":"While most publicly available object detection models are trained on the COCO dataset (which includes 80 object classes), this library supports models trained on any dataset. For instance, you can use the Ultralytics Open Images V7 model that can detect over 600 different object classes. The library automatically adapts to different models - simply specify the appropriate ONNX model and classes path, along with the correct model implementation (e.g., `YOLO.Models.Ultralytics` for Ultralytics models).\n\nFor a complete example, see the [Ultralytics YOLOv8 - Open Images V7](examples/yolo_oiv7.livemd) livebook.\n\n![YOLOv8x - Open Images V7](guides/images/yolov8_oiv7.jpg)","ref":"readme.html#models-trained-on-different-datasets"},{"type":"extras","title":"Livebook Examples - README","doc":"* [Ultralytics YOLO](examples/ultralytics_yolo.livemd): Demonstrates the complete workflow of using Ultralytics YOLOv11, from downloading and converting the model to ONNX format using Python, to running object detection on a traffic scene image containing multiple objects like cars, bicycles, and pedestrians.\n* [YOLOX](examples/yolox.livemd): Shows how to use YOLOX, an alternative YOLO implementation that offers similar performance but with a more permissive Apache License 2.0 (compared to Ultralytics' AGPL license). Includes downloading pre-converted ONNX models and running inference directly.\n* [Ultralytics YOLOv8 - Open Images V7](examples/yolo_oiv7.livemd): While the previous examples use models trained on COCO's 80 classes, this example demonstrates how to use a model trained on a different dataset. It shows how to run Ultralytics YOLOv8x, an extra-large model trained on the Open Images V7 dataset that can detect over 600 different object classes.\n* [Real-time Object Detection](examples/webcam.livemd): A livebook example showing how to perform real-time object detection using your webcam, with continuous frame processing and real-time visualization of detected objects.\n\n## [Performance Benchmarks](guides/benchmarks.md) - See how YOLO performs on different models and devices","ref":"readme.html#livebook-examples"},{"type":"extras","title":"Under the Hood - README","doc":"See [Under the Hood](guides/under_the_hood.md) for a detailed explanation of how YOLO works internally, including preprocessing, model inference, and postprocessing steps.","ref":"readme.html#under-the-hood"},{"type":"extras","title":"Ultralytics to ONNX","doc":"# Ultralytics to ONNX\n\nDownload the PyTorch model from the [Ultralytics assets repository](https://github.com/ultralytics/assets/releases/tag/v8.3.0). At the moment, the latest version is YOLOv11, available in various sizes: `yolov11n.pt` (nano), `yolov11s.pt` (small), `yolov11m.pt` (medium), etc. For detailed specifications of each model variant, visit the [Ultralytics documentation](https://docs.ultralytics.com/tasks/detect/#models).\n\n\nI've prepared a python [`ultralytics_to_onnx.py` script](python/ultralytics_to_onnx.py) to easily download and convert Ultralytics `pt` models to the right `onnx` format.\n\nTo run it, first, you need to install the dependencies (`requests` and `ultralytics`)\n```bash\npip install -r python/requirements.txt\n```\n\nThen, run the script by specifying the model size, such as `n`:\n\n```bash\npython python/ultralytics_to_onnx.py yolo11n\n```\n\nThe script will download the `.pt` model and generate two files:\n\n* `models/yolo11n.onnx`: the Yolo11n model with weights\n* `models/yolo11n_classes.json`: the list of object classes","ref":"ultralytics_to_onnx.html"},{"type":"extras","title":"Under the Hood","doc":"# Under the Hood\nLet's see how `YOLO.detect/3` works.","ref":"under_the_hood.html"},{"type":"extras","title":"Load Yolo11n Model - Under the Hood","doc":"Loads the *Yolo11n* model using the `model_path` and `classes_path`. Optionally, specify `model_impl`, which defaults to `YOLO.Models.Ultralytics`.\n\n```elixir\nmodel = YOLO.load([\n  model_path: \"models/yolo11n.onnx\", \n  classes_path: \"models/yolo11n_classes.json\"\n])\n```","ref":"under_the_hood.html#load-yolo11n-model"},{"type":"extras","title":"Preprocessing - Under the Hood","doc":"```elixir\nmat = Evision.imread(image_path)\n\n{input_tensor, scaling_config} = YOLO.Models.Ultralytics.preprocess(model, mat, [frame_scaler: YOLO.FrameScalers.EvisionScaler])\n```\n\nBefore running object detection, the input image needs to be preprocessed to match the model's expected input format. The preprocessing steps are:\n\n1. **Resize and Pad Image to 640x640**\n   - The image is resized while preserving aspect ratio to fit within 640x640 pixels\n   - Any remaining space is padded with gray color (value 114) to reach exactly 640x640\n   - This is handled by the `FrameScaler` behaviour and its implementations\n\n2. **Convert to Normalized Tensor**\n   - The image is converted to an Nx tensor with shape `{1, 3, 640, 640}`\n   - Pixel values are normalized from `0-255` to `0.0-1.0` range\n   - The channels are reordered from `RGB` to the model's expected format (`BGR` in this case)\n\nThe `FrameScaler` behaviour provides a consistent interface for handling different image formats:\n\n- `EvisionScaler` - For OpenCV Mat images from Evision\n- `ImageScaler` - For images using the Image library  \n- `NxIdentityScaler` - For ready to use Nx tensors","ref":"under_the_hood.html#preprocessing"},{"type":"extras","title":"Run Object Detection - Under the Hood","doc":"Then run the detection by passing the `model` and the image tensor `input_tensor`.\n\n```elixir\n# input_tensor {1, 3, 640, 640}\noutput_tensor = YOLO.Models.run(model, input_tensor)\n# output_tensor {1, 84, 8400}\n```\n\nYou can also adjust detection thresholds (`iou_threshold` and `prob_threshold`, which both default to `0.45` and `0.25` respectively) using the third argument.","ref":"under_the_hood.html#run-object-detection"},{"type":"extras","title":"Postprocessing - Under the Hood","doc":"```elixir\nresult_rows = YOLO.Models.Ultralytics.postprocess(model, output_tensor, scaling_config, opts)\n```\nwhere `result_rows` is a list of lists, where each inner list represents a detected object with 6 elements:\n\n```elixir\n[\n  [cx, cy, w, h, prob, class_idx],\n  ...\n]\n```\n\nThe model's raw output needs to be post-processed to extract meaningful detections. For YOLOv8n, the `output_tensor` has shape `{1, 84, 8400}` where:\n\n- 84 represents 4 bbox coordinates + 80 class probabilities\n- 8400 represents the number of candidate detections\n\nThe postprocessing steps are:\n\n1. **Filter Low Probability Detections**\n   - Each detection has probabilities for all classes\n   - Only keep detections where max class probability exceeds `prob_threshold` (default 0.25)\n\n2. **Non-Maximum Suppression (NMS)**\n   - Remove overlapping boxes for the same object\n   - For each class, compare boxes using Intersection over Union (IoU)\n   - If IoU > `iou_threshold` (default 0.45), keep only highest probability box\n   - This prevents multiple detections of the same object\n\n3. **Scale Coordinates**\n   - The detected coordinates are based on the model's 640x640 input\n   - Use the `scaling_config` from preprocessing to map back to original image size\n   - This accounts for any resizing/padding done during preprocessing","ref":"under_the_hood.html#postprocessing"},{"type":"extras","title":"Convert Detections to Structured Maps - Under the Hood","doc":"Finally, convert the raw detection results into structured maps containing bounding box coordinates, class labels, and probabilities:\n```elixir\niex> YOLO.to_detected_objects(result_rows, model.classes)\n[\n  %{\n    class: \"person\",\n    prob: 0.57,\n    bbox: %{h: 126, w: 70, cx: 700, cy: 570},\n    class_idx: 0\n  },\n  ...\n]\n```","ref":"under_the_hood.html#convert-detections-to-structured-maps"},{"type":"extras","title":"Render results on the image - Under the Hood","doc":"To visualize the detection results on the image, we can use the `KinoYOLO.Draw.draw_detected_objects/2` function. This function takes an `Image` and a list of detected objects, and returns a new image with bounding boxes and labels drawn on it.","ref":"under_the_hood.html#render-results-on-the-image"},{"type":"extras","title":"Ultralytics Yolo11 - CPU","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Ultralytics Yolo11 - CPU\n\n```elixir\nMix.install(\n  [\n    {:yolo, \">= 0.0.0\"},\n    {:nx, \"~> 0.9\"},\n    {:exla, \"~> 0.9\"},\n    {:image, \"~> 0.54\"},\n    {:evision, \"~> 0.2\"},\n    {:kino, \"~> 0.16\"},\n    {:kino_yolo, github: \"poeticoding/kino_yolo\", branch: \"main\"},\n    {:pythonx, \"~> 0.4.2\"},\n    {:kino_pythonx, \"~> 0.1.0\"}\n  ],\n  config: [\n    nx: [default_backend: EXLA.Backend]\n  ]\n)\n\nApplication.put_env(:ortex, Ortex.Native, features: [:cpu])\n```\n\n```pyproject.toml\n[project]\nname = \"ultralytics_to_onnx\"\nversion = \"0.0.0\"\nrequires-python = \"==3.13.*\"\ndependencies = [\n  \"ultralytics==8.3.155\",\n  \"onnx==1.18.0\",\n  \"onnxruntime==1.22.0\",\n  \"onnx_pytorch==0.1.5\",\n  \"requests\"\n]\n```","ref":"ultralytics_yolo.html"},{"type":"extras","title":"Define model and paths - Ultralytics Yolo11 - CPU","doc":"```elixir\nmodel_name = \"yolo11m\"\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\"yolo11m\"\n```\n\n```elixir\nimage_path = \"benchmarks/images/traffic.jpg\"\nmodel_path = \"#{model_name}.onnx\"\nclasses_path = \"models/coco_classes.json\"\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"ultralytics_yolo.html#define-model-and-paths"},{"type":"extras","title":"Ultralytics Yolo11 to ONNX - Ultralytics Yolo11 - CPU","doc":"```elixir\nPythonx.eval(\n  \"\"\"\n  from ultralytics import YOLO\n  IMAGE_SIZE = 640\n  model_name = model_name.decode(\"utf-8\")\n  \n  model = YOLO(model_name)\n  model.export(format='onnx', imgsz=IMAGE_SIZE, opset=12)\n  \"\"\",\n  %{\"model_name\" => model_name}\n)\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nDownloading https://github.com/ultralytics/assets/releases/download/v8.3.0/yolo11m.pt to 'yolo11m.pt'...\nUltralytics 8.3.155 🚀 Python-3.13.1 torch-2.7.1 CPU (Apple M3)\nYOLO11m summary (fused): 125 layers, 20,091,712 parameters, 0 gradients, 68.0 GFLOPs\n\nPyTorch: starting from 'yolo11m.pt' with input shape (1, 3, 640, 640) BCHW and output shape(s) (1, 84, 8400) (38.8 MB)\nrequirements: Ultralytics requirements ['onnx>=1.12.0,<1.18.0', 'onnxslim>=0.1.56'] not found, attempting AutoUpdate...\nRequirement already satisfied: onnx<1.18.0,>=1.12.0 in ./venv/lib/python3.10/site-packages (1.17.0)\nRequirement already satisfied: onnxslim>=0.1.56 in ./venv/lib/python3.10/site-packages (0.1.57)\nRequirement already satisfied: protobuf>=3.20.2 in ./venv/lib/python3.10/site-packages (from onnx<1.18.0,>=1.12.0) (6.31.1)\nRequirement already satisfied: numpy>=1.20 in ./venv/lib/python3.10/site-packages (from onnx<1.18.0,>=1.12.0) (2.2.6)\nRequirement already satisfied: packaging in ./venv/lib/python3.10/site-packages (from onnxslim>=0.1.56) (25.0)\nRequirement already satisfied: sympy in ./venv/lib/python3.10/site-packages (from onnxslim>=0.1.56) (1.14.0)\nRequirement already satisfied: mpmath<1.4,>=1.1.0 in ./venv/lib/python3.10/site-packages (from sympy->onnxslim>=0.1.56) (1.3.0)\n\nrequirements: AutoUpdate success ✅ 0.8s\nWARNING ⚠️ requirements: Restart runtime or rerun command for updates to take effect\n\n\nONNX: starting export with onnx 1.18.0 opset 12...\nWARNING ⚠️ ONNX: simplifier failure: No module named 'onnxslim'\nONNX: export success ✅ 1.4s, saved as 'yolo11m.onnx' (76.9 MB)\n\nExport complete (1.9s)\nResults saved to /Users/alvise/code/yolo_elixir\nPredict:         yolo predict task=detect model=yolo11m.onnx imgsz=640  \nValidate:        yolo val task=detect model=yolo11m.onnx imgsz=640 data=/ultralytics/ultralytics/cfg/datasets/coco.yaml  \nVisualize:       https://netron.app\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"ultralytics_yolo.html#ultralytics-yolo11-to-onnx"},{"type":"extras","title":"Load image - Ultralytics Yolo11 - CPU","doc":"```elixir\nmat = Evision.imread(image_path)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Evision.Mat{\n  channels: 3,\n  dims: 2,\n  type: {:u, 8},\n  raw_type: 16,\n  shape: {1080, 1920, 3},\n  ref: #Reference<0.2594270970.3468558359.70281>\n}\n```","ref":"ultralytics_yolo.html#load-image"},{"type":"extras","title":"Load and run YOLO - Ultralytics Yolo11 - CPU","doc":"```elixir\nmodel = YOLO.load(model_path: model_path, classes_path: classes_path, eps: [:cpu])\n\ndetected_objects = \n  model\n  |> YOLO.detect(mat, iou_threshold: 0.45, prob_threshold: 0.25)\n  |> YOLO.to_detected_objects(model.classes)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n17:41:08.764 [info] Loaded model yolo11m.onnx with [:cpu] execution providers\n\n17:41:08.765 [info] Initialized model yolo11m.onnx\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  %{\n    class: \"person\",\n    prob: 0.3464750051498413,\n    bbox: %{h: 95, w: 47, cx: 1675, cy: 391},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.34952861070632935,\n    bbox: %{h: 160, w: 57, cx: 1768, cy: 501},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.36030319333076477,\n    bbox: %{h: 157, w: 55, cx: 355, cy: 462},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.5312483310699463,\n    bbox: %{h: 110, w: 50, cx: 695, cy: 386},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.5382186770439148,\n    bbox: %{h: 155, w: 64, cx: 803, cy: 570},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.5533820390701294,\n    bbox: %{h: 165, w: 76, cx: 44, cy: 521},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.5973995923995972,\n    bbox: %{h: 141, w: 43, cx: 1617, cy: 421},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.6149826049804688,\n    bbox: %{h: 128, w: 42, cx: 1533, cy: 413},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.6478251814842224,\n    bbox: %{h: 120, w: 44, cx: 556, cy: 403},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.7199457883834839,\n    bbox: %{h: 157, w: 52, cx: 1685, cy: 491},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.7302370667457581,\n    bbox: %{h: 197, w: 94, cx: 733, cy: 687},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.7496381402015686,\n    bbox: %{h: 130, w: 69, cx: 701, cy: 567},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.7714580297470093,\n    bbox: %{h: 156, w: 64, cx: 302, cy: 558},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.7862504720687866,\n    bbox: %{h: 222, w: 90, cx: 469, cy: 846},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.7874720096588135,\n    bbox: %{h: 238, w: 81, cx: 41, cy: 801},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.8138344883918762,\n    bbox: %{h: 233, w: 101, cx: 679, cy: 848},\n    class_idx: 0\n  },\n  %{\n    class: \"person\",\n    prob: 0.8157120943069458,\n    bbox: %{h: 200, w: 81, cx: 605, cy: 768},\n    class_idx: 0\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.4413682520389557,\n    bbox: %{h: 83, w: 32, cx: 803, cy: 625},\n    class_idx: 1\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.4464598298072815,\n    bbox: %{h: 135, w: 56, cx: 599, cy: 849},\n    class_idx: 1\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.4616367518901825,\n    bbox: %{h: 182, w: 56, cx: 675, cy: 925},\n    class_idx: 1\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.5315952897071838,\n    bbox: %{h: 75, w: 42, cx: 694, cy: 416},\n    class_idx: 1\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.6614882349967957,\n    bbox: %{h: 165, w: 69, cx: 464, cy: 955},\n    class_idx: 1\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.6782569885253906,\n    bbox: %{h: 113, w: 63, cx: 725, cy: 737},\n    class_idx: 1\n  },\n  %{class: \"car\", prob: 0.35281914472579956, bbox: %{h: 35, w: 57, cx: 1090, cy: 18}, class_idx: 2},\n  %{\n    class: \"car\",\n    prob: 0.4985942840576172,\n    bbox: %{h: 245, w: 224, cx: 1098, cy: 474},\n    class_idx: 2\n  },\n  %{class: \"car\", prob: 0.5132410526275635, bbox: %{h: 64, w: 84, cx: 1034, cy: 167}, class_idx: 2},\n  %{class: \"car\", prob: 0.5697806477546692, bbox: %{h: 71, w: 79, cx: 1172, cy: 149}, class_idx: 2},\n  %{class: \"car\", prob: 0.6225624084472656, bbox: %{h: 49, w: 80, cx: 1193, cy: 187}, class_idx: 2},\n  %{class: \"car\", prob: 0.652853786945343, bbox: %{h: 86, w: 97, cx: 1024, cy: 212}, class_idx: 2},\n  %{class: \"car\", prob: 0.7560621500015259, bbox: %{h: 81, w: 105, cx: 1203, cy: 238}, class_idx: 2},\n  %{\n    class: \"car\",\n    prob: 0.8117802143096924,\n    bbox: %{h: 110, w: 129, cx: 1254, cy: 324},\n    class_idx: 2\n  },\n  %{class: \"car\", prob: 0.8485523462295532, bbox: %{h: 88, w: 102, cx: 1038, cy: 268}, class_idx: 2},\n  %{\n    class: \"car\",\n    prob: 0.8879761695861816,\n    bbox: %{h: 121, w: 148, cx: 1301, cy: 402},\n    class_idx: 2\n  },\n  %{\n    class: \"motorcycle\",\n    prob: 0.2533663511276245,\n    bbox: %{h: 43, w: 28, cx: 866, cy: 228},\n    class_idx: 3\n  },\n  %{\n    class: \"truck\",\n    prob: 0.580411970615387,\n    bbox: %{h: 246, w: 223, cx: 1098, cy: 474},\n    class_idx: 7\n  },\n  %{\n    class: \"traffic light\",\n    prob: 0.30687159299850464,\n    bbox: %{h: 46, w: 44, cx: 1478, cy: 225},\n    class_idx: 9\n  },\n  %{\n    class: \"traffic light\",\n    prob: 0.4421222507953644,\n    bbox: %{h: 137, w: 75, cx: 864, cy: 113},\n    class_idx: 9\n  },\n  %{\n    class: \"traffic light\",\n    prob: 0.6856074929237366,\n    bbox: %{h: 104, w: 38, cx: 195, cy: 387},\n    class_idx: 9\n  },\n  %{\n    class: \"traffic light\",\n    prob: 0.7127625346183777,\n    bbox: %{h: 141, w: 77, cx: 1333, cy: 103},\n    class_idx: 9\n  },\n  %{\n    class: \"backpack\",\n    prob: 0.3877813518047333,\n    bbox: %{h: 67, w: 35, cx: 449, cy: 951},\n    class_idx: 24\n  },\n  %{\n    class: \"backpack\",\n    prob: 0.6103652119636536,\n    bbox: %{h: 58, w: 51, cx: 719, cy: 646},\n    class_idx: 24\n  },\n  %{\n    class: \"backpack\",\n    prob: 0.6811151504516602,\n    bbox: %{h: 76, w: 55, cx: 28, cy: 759},\n    class_idx: 24\n  },\n  %{\n    class: \"handbag\",\n    prob: 0.3150494694709778,\n    bbox: %{h: 45, w: 51, cx: 1768, cy: 488},\n    class_idx: 26\n  }\n]\n```\n\n```elixir\n{:ok, image} = Image.from_evision(mat)\nKinoYOLO.Draw.draw_detected_objects(image, detected_objects)\n```","ref":"ultralytics_yolo.html#load-and-run-yolo"},{"type":"extras","title":"YOLOX","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# YOLOX\n\n```elixir\nMix.install(\n  [\n    {:yolo, \">= 0.0.0\"},\n    {:nx, \"~> 0.9\"},\n    {:exla, \"~> 0.9\"},\n    {:image, \"~> 0.54\"},\n    {:evision, \"~> 0.2\"},\n    {:req, \"~> 0.5.10\"},\n    {:kino, \"~> 0.16\"},\n    {:kino_yolo, github: \"poeticoding/kino_yolo\", branch: \"main\"},\n  ],\n  config: [\n    nx: [default_backend: EXLA.Backend]\n  ]\n)\n\nApplication.put_env(:ortex, Ortex.Native, features: [:cpu])\n```","ref":"yolox.html"},{"type":"extras","title":"Download YOLOX model - YOLOX","doc":"```elixir\nmodel_name = \"yolox_m\"\n\n#https://github.com/Megvii-BaseDetection/YOLOX/tree/main/demo/ONNXRuntime\nbase_url = \"https://github.com/Megvii-BaseDetection/YOLOX/releases/download/0.1.1rc0/\"\nmodel_filename = \"#{model_name}.onnx\"\n\n%{body: data} = Req.get!(\"#{base_url}/#{model_filename}\")\nFile.write!(\"#{model_name}.onnx\", data)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n17:44:11.158 [debug] redirecting to https://objects.githubusercontent.com/github-production-release-asset-2e65be/386811486/2dfe0f1c-c4e2-4697-ad7c-46cb04f404f3?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=releaseassetproduction%2F20250617%2Fus-east-1%2Fs3%2Faws4_request&X-Amz-Date=20250617T154411Z&X-Amz-Expires=300&X-Amz-Signature=9553822045bab5a16103fb8644b91277a53c2b52c8397020272c2702d1cd2971&X-Amz-SignedHeaders=host&response-content-disposition=attachment%3B%20filename%3Dyolox_m.onnx&response-content-type=application%2Foctet-stream\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"yolox.html#download-yolox-model"},{"type":"extras","title":"Load and Run - YOLOX","doc":"```elixir\nmat = Evision.imread(\"benchmarks/images/traffic.jpg\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Evision.Mat{\n  channels: 3,\n  dims: 2,\n  type: {:u, 8},\n  raw_type: 16,\n  shape: {1080, 1920, 3},\n  ref: #Reference<0.1897054955.4006215703.115351>\n}\n```\n\n```elixir\nmodel = YOLO.load(\n  model_path: model_filename, \n  model_impl: YOLO.Models.YOLOX,\n  classes_path: \"models/coco_classes.json\",\n  eps: [:cpu]\n)\n\ndetected_objects = \n  model\n  |> YOLO.detect(mat, iou_threshold: 0.45, prob_threshold: 0.25)\n  |> YOLO.to_detected_objects(model.classes)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n17:44:17.820 [info] Loaded model yolox_m.onnx with [:cpu] execution providers\n\n17:44:17.917 [info] Initialized model yolox_m.onnx\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  %{\n    class: \"person\",\n    prob: 0.2730484902858734,\n    class_idx: 0,\n    bbox: %{h: 71, w: 33, cx: 867, cy: 209}\n  },\n  %{\n    class: \"person\",\n    prob: 0.2760665714740753,\n    class_idx: 0,\n    bbox: %{h: 109, w: 37, cx: 1704, cy: 339}\n  },\n  %{\n    class: \"person\",\n    prob: 0.42914506793022156,\n    class_idx: 0,\n    bbox: %{h: 125, w: 39, cx: 1743, cy: 420}\n  },\n  %{\n    class: \"person\",\n    prob: 0.4699484407901764,\n    class_idx: 0,\n    bbox: %{h: 84, w: 46, cx: 1675, cy: 387}\n  },\n  %{\n    class: \"person\",\n    prob: 0.5709174871444702,\n    class_idx: 0,\n    bbox: %{h: 108, w: 45, cx: 696, cy: 389}\n  },\n  %{\n    class: \"person\",\n    prob: 0.6232989430427551,\n    class_idx: 0,\n    bbox: %{h: 163, w: 75, cx: 46, cy: 520}\n  },\n  %{\n    class: \"person\",\n    prob: 0.6443946361541748,\n    class_idx: 0,\n    bbox: %{h: 118, w: 40, cx: 1502, cy: 360}\n  },\n  %{\n    class: \"person\",\n    prob: 0.657820999622345,\n    class_idx: 0,\n    bbox: %{h: 149, w: 55, cx: 1768, cy: 505}\n  },\n  %{\n    class: \"person\",\n    prob: 0.6603461503982544,\n    class_idx: 0,\n    bbox: %{h: 163, w: 54, cx: 352, cy: 457}\n  },\n  %{\n    class: \"person\",\n    prob: 0.740202009677887,\n    class_idx: 0,\n    bbox: %{h: 149, w: 59, cx: 802, cy: 571}\n  },\n  %{\n    class: \"person\",\n    prob: 0.7521814703941345,\n    class_idx: 0,\n    bbox: %{h: 118, w: 43, cx: 557, cy: 403}\n  },\n  %{\n    class: \"person\",\n    prob: 0.7617838382720947,\n    class_idx: 0,\n    bbox: %{h: 230, w: 98, cx: 673, cy: 851}\n  },\n  %{\n    class: \"person\",\n    prob: 0.7718674540519714,\n    class_idx: 0,\n    bbox: %{h: 128, w: 41, cx: 1533, cy: 412}\n  },\n  %{\n    class: \"person\",\n    prob: 0.7752799391746521,\n    class_idx: 0,\n    bbox: %{h: 151, w: 72, cx: 699, cy: 578}\n  },\n  %{\n    class: \"person\",\n    prob: 0.7786338925361633,\n    class_idx: 0,\n    bbox: %{h: 138, w: 45, cx: 1616, cy: 420}\n  },\n  %{\n    class: \"person\",\n    prob: 0.7892401218414307,\n    class_idx: 0,\n    bbox: %{h: 187, w: 94, cx: 731, cy: 682}\n  },\n  %{\n    class: \"person\",\n    prob: 0.7999732494354248,\n    class_idx: 0,\n    bbox: %{h: 240, w: 82, cx: 40, cy: 797}\n  },\n  %{\n    class: \"person\",\n    prob: 0.8141907453536987,\n    class_idx: 0,\n    bbox: %{h: 157, w: 69, cx: 299, cy: 559}\n  },\n  %{\n    class: \"person\",\n    prob: 0.8287546634674072,\n    class_idx: 0,\n    bbox: %{h: 161, w: 50, cx: 1687, cy: 491}\n  },\n  %{\n    class: \"person\",\n    prob: 0.8520509004592896,\n    class_idx: 0,\n    bbox: %{h: 206, w: 89, cx: 606, cy: 771}\n  },\n  %{\n    class: \"person\",\n    prob: 0.8624670505523682,\n    class_idx: 0,\n    bbox: %{h: 256, w: 92, cx: 470, cy: 865}\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.4504697024822235,\n    class_idx: 1,\n    bbox: %{h: 70, w: 37, cx: 695, cy: 419}\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.5685099959373474,\n    class_idx: 1,\n    bbox: %{h: 135, w: 66, cx: 466, cy: 969}\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.6224793791770935,\n    class_idx: 1,\n    bbox: %{h: 82, w: 37, cx: 801, cy: 622}\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.6391600966453552,\n    class_idx: 1,\n    bbox: %{h: 151, w: 61, cx: 590, cy: 852}\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.6717647314071655,\n    class_idx: 1,\n    bbox: %{h: 109, w: 66, cx: 725, cy: 737}\n  },\n  %{\n    class: \"bicycle\",\n    prob: 0.706743597984314,\n    class_idx: 1,\n    bbox: %{h: 164, w: 59, cx: 671, cy: 932}\n  },\n  %{class: \"car\", prob: 0.3101554811000824, class_idx: 2, bbox: %{h: 39, w: 59, cx: 1028, cy: 67}},\n  %{class: \"car\", prob: 0.3179891109466553, class_idx: 2, bbox: %{h: 58, w: 85, cx: 1027, cy: 115}},\n  %{class: \"car\", prob: 0.540040135383606, class_idx: 2, bbox: %{h: 58, w: 81, cx: 1032, cy: 163}},\n  %{class: \"car\", prob: 0.6538744568824768, class_idx: 2, bbox: %{h: 36, w: 55, cx: 1091, cy: 17}},\n  %{class: \"car\", prob: 0.7471896409988403, class_idx: 2, bbox: %{h: 43, w: 74, cx: 1192, cy: 183}},\n  %{class: \"car\", prob: 0.7499881386756897, class_idx: 2, bbox: %{h: 75, w: 75, cx: 1168, cy: 149}},\n  %{class: \"car\", prob: 0.7761209607124329, class_idx: 2, bbox: %{h: 84, w: 91, cx: 1027, cy: 214}},\n  %{\n    class: \"car\",\n    prob: 0.8965244293212891,\n    class_idx: 2,\n    bbox: %{h: 107, w: 122, cx: 1249, cy: 324}\n  },\n  %{class: \"car\", prob: 0.9002096056938171, class_idx: 2, bbox: %{h: 85, w: 103, cx: 1203, cy: 239}},\n  %{class: \"car\", prob: 0.9072261452674866, class_idx: 2, bbox: %{h: 92, w: 99, cx: 1040, cy: 267}},\n  %{\n    class: \"car\",\n    prob: 0.9226636290550232,\n    class_idx: 2,\n    bbox: %{h: 121, w: 146, cx: 1303, cy: 402}\n  },\n  %{\n    class: \"truck\",\n    prob: 0.3016549050807953,\n    class_idx: 7,\n    bbox: %{h: 60, w: 86, cx: 1027, cy: 118}\n  },\n  %{\n    class: \"truck\",\n    prob: 0.8140892386436462,\n    class_idx: 7,\n    bbox: %{h: 246, w: 214, cx: 1093, cy: 473}\n  },\n  %{\n    class: \"traffic light\",\n    prob: 0.5662282109260559,\n    class_idx: 9,\n    bbox: %{h: 140, w: 81, cx: 1332, cy: 103}\n  },\n  %{\n    class: \"traffic light\",\n    prob: 0.6133936643600464,\n    class_idx: 9,\n    bbox: %{h: 134, w: 79, cx: 865, cy: 113}\n  },\n  %{\n    class: \"traffic light\",\n    prob: 0.6605126857757568,\n    class_idx: 9,\n    bbox: %{h: 106, w: 43, cx: 195, ...}\n  },\n  %{class: \"backpack\", prob: 0.3358326852321625, class_idx: 24, bbox: %{h: 77, w: 49, ...}},\n  %{class: \"backpack\", prob: 0.3416420519351959, class_idx: 24, bbox: %{h: 86, ...}}\n]\n```\n\n```elixir\n{:ok, image} = Image.from_evision(mat)\nKinoYOLO.Draw.draw_detected_objects(image, detected_objects)\n```","ref":"yolox.html#load-and-run"},{"type":"extras","title":"Real-time Object Detection","doc":"# Real-time Object Detection\n\n```elixir\nMix.install(\n  [\n    {:yolo, \">= 0.0.0\"},\n    {:nx, \"~> 0.9\"},\n    {:exla, \"~> 0.9\"},\n    {:image, \"~> 0.54\"},\n    {:evision, \"~> 0.2\"},\n    {:kino, \"~> 0.16\"},\n    {:kino_yolo, github: \"poeticoding/kino_yolo\", branch: \"main\"},\n    {:pythonx, \"~> 0.4.2\"},\n    {:kino_pythonx, \"~> 0.1.0\"}\n  ],\n  config: [\n    nx: [default_backend: EXLA.Backend]\n  ]\n)\n\nApplication.put_env(:ortex, Ortex.Native, features: [:cpu])\n```\n\n```pyproject.toml\n[project]\nname = \"realtime_object_detection\"\nversion = \"0.0.0\"\nrequires-python = \"==3.13.*\"\ndependencies = [\n  \"ultralytics==8.3.155\",\n  \"onnx==1.18.0\",\n  \"onnxruntime==1.22.0\",\n  \"onnx_pytorch==0.1.5\",\n  \"requests\"\n]\n```","ref":"webcam.html"},{"type":"extras","title":"Define model and paths - Real-time Object Detection","doc":"```elixir\nmodel_name = \"yolo11n\"\n\nmodel_path = \"#{model_name}.onnx\"\nclasses_path = \"models/coco_classes.json\"\n:ok\n```","ref":"webcam.html#define-model-and-paths"},{"type":"extras","title":"Ultralytics Yolo11 to ONNX - Real-time Object Detection","doc":"```elixir\nPythonx.eval(\n  \"\"\"\n  from ultralytics import YOLO\n  IMAGE_SIZE = 640\n  model_name = model_name.decode(\"utf-8\")\n  \n  model = YOLO(model_name)\n  model.export(format='onnx', imgsz=IMAGE_SIZE, opset=12)\n  \"\"\",\n  %{\"model_name\" => model_name}\n)\n:ok\n```","ref":"webcam.html#ultralytics-yolo11-to-onnx"},{"type":"extras","title":"Load model - Real-time Object Detection","doc":"```elixir\nmodel = YOLO.load(model_path: model_path, classes_path: classes_path)\n```","ref":"webcam.html#load-model"},{"type":"extras","title":"Load WebCam with Evision - Real-time Object Detection","doc":"```elixir\ncamera = Evision.VideoCapture.videoCapture(0)\n```\n\n```elixir\nEvision.VideoCapture.read(camera)\n```","ref":"webcam.html#load-webcam-with-evision"},{"type":"extras","title":"Run Object Detection and render detections on the frame - Real-time Object Detection","doc":"```elixir\nStream.repeatedly(fn -> \n  mat = Evision.VideoCapture.read(camera)\n\n  detected_objects =\n    model\n    |> YOLO.detect(mat)\n    |> YOLO.to_detected_objects(model.classes)\n  \n  {mat, detected_objects}\nend)\n|> Kino.animate(fn {mat, detected_objects} ->\n  {:ok, image} = Image.from_evision(mat)\n  KinoYOLO.Draw.draw_detected_objects(image, detected_objects)\nend)\n```","ref":"webcam.html#run-object-detection-and-render-detections-on-the-frame"},{"type":"extras","title":"Ultralytics YOLOv8 - Open Images V7","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Ultralytics YOLOv8 - Open Images V7\n\n```elixir\nMix.install(\n  [\n    {:yolo, \">= 0.0.0\"},\n    {:nx, \"~> 0.9\"},\n    {:exla, \"~> 0.9\"},\n    {:image, \"~> 0.54\"},\n    {:evision, \"~> 0.2\"},\n    {:kino, \"~> 0.16\"},\n    {:kino_yolo, github: \"poeticoding/kino_yolo\", branch: \"main\"},\n    {:pythonx, \"~> 0.4.2\"},\n    {:kino_pythonx, \"~> 0.1.0\"}\n  ],\n  config: [\n    nx: [default_backend: EXLA.Backend]\n  ]\n)\n\nApplication.put_env(:ortex, Ortex.Native, features: [:cpu])\n```\n\n```pyproject.toml\n[project]\nname = \"ultralytics_to_onnx\"\nversion = \"0.0.0\"\nrequires-python = \"==3.13.*\"\ndependencies = [\n  \"ultralytics==8.3.155\",\n  \"onnx==1.18.0\",\n  \"onnxruntime==1.22.0\",\n  \"onnx_pytorch==0.1.5\",\n  \"requests\"\n]\n```","ref":"yolo_oiv7.html"},{"type":"extras","title":"Ultralytics yolov8l-oiv7 to ONNX - Ultralytics YOLOv8 - Open Images V7","doc":"```elixir\n# https://docs.ultralytics.com/datasets/detect/open-images-v7/\nmodel_name = \"yolov8x-oiv7\"\n\nimage_path = \"benchmarks/images/table.jpg\"\nmodel_path = \"#{model_name}.onnx\"\nclasses_path = \"#{model_name}_classes.json\"\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\n```elixir\nPythonx.eval(\n  \"\"\"\n  from ultralytics import YOLO\n  import json\n  \n  IMAGE_SIZE = 640\n  model_name = model_name.decode(\"utf-8\")\n  \n  model = YOLO(model_name)\n  model.export(format='onnx', imgsz=IMAGE_SIZE, opset=12)\n\n  # Export the categories\n  with open(classes_path, \"w\") as f:\n    data = [model.names[idx] for idx in model.names]\n    json.dump(data, f)\n  \"\"\",\n  %{\"model_name\" => model_name, \"classes_path\" => classes_path}\n)\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nnil\n```","ref":"yolo_oiv7.html#ultralytics-yolov8l-oiv7-to-onnx"},{"type":"extras","title":"Load and Run - Ultralytics YOLOv8 - Open Images V7","doc":"```elixir\nmat = Evision.imread(image_path)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%Evision.Mat{\n  channels: 3,\n  dims: 2,\n  type: {:u, 8},\n  raw_type: 16,\n  shape: {1138, 640, 3},\n  ref: #Reference<0.2823733541.3473539090.13647>\n}\n```\n\n```elixir\nmodel = YOLO.load(model_path: model_path, classes_path: classes_path, eps: [:cpu])\n\ndetected_objects = \n  model\n  |> YOLO.detect(mat, iou_threshold: 0.45, prob_threshold: 0.25)\n  |> YOLO.to_detected_objects(model.classes)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n18:27:35.461 [info] Loaded model yolov8x-oiv7.onnx with [:cpu] execution providers\n\n18:27:35.461 [info] Initialized model yolov8x-oiv7.onnx\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  %{\n    class: \"Food\",\n    prob: 0.2534943222999573,\n    bbox: %{h: 38, w: 73, cx: 257, cy: 468},\n    class_idx: 199\n  },\n  %{\n    class: \"Food\",\n    prob: 0.3215067982673645,\n    bbox: %{h: 72, w: 134, cx: 512, cy: 515},\n    class_idx: 199\n  },\n  %{\n    class: \"Food\",\n    prob: 0.35645782947540283,\n    bbox: %{h: 35, w: 99, cx: 351, cy: 385},\n    class_idx: 199\n  },\n  %{\n    class: \"Food\",\n    prob: 0.5081610679626465,\n    bbox: %{h: 70, w: 117, cx: 389, cy: 484},\n    class_idx: 199\n  },\n  %{\n    class: \"Kitchen appliance\",\n    prob: 0.2527419328689575,\n    bbox: %{h: 45, w: 64, cx: 38, cy: 33},\n    class_idx: 291\n  },\n  %{\n    class: \"Table\",\n    prob: 0.7417454123497009,\n    bbox: %{h: 896, w: 640, cx: 320, cy: 673},\n    class_idx: 514\n  }\n]\n```\n\n```elixir\n{:ok, image} = Image.from_evision(mat)\nKinoYOLO.Draw.draw_detected_objects(image, detected_objects)\n```","ref":"yolo_oiv7.html#load-and-run"},{"type":"extras","title":"Benchmarks","doc":"# Benchmarks\n\nBelow you find some Ultralytics `yolo11` benchmarks of **full pipeline** (preprocess, run, postprocess with NMS). \nYou can run benchmarks yourself on your machine with\n\n```bash\nmix run benchmarks/ultralytics_yolo.exs    \n```\n\n*   is the mandatory first argument that should be one of: `cpu`, `coreml`, `directml`, `cuda`, or `tensorrt`. \n*   is the required path to your ONNX file.\n\nTo set the proper Nx acceleration, configure your `config/config.exs` file with the appropriate backend.\n\n\n* [Benchmarks on MacBook Air M3](benchmarks/macbook_air_m3.md)","ref":"benchmarks.html"},{"type":"extras","title":"Benchmarks on MacBook Air M3","doc":"# Benchmarks on MacBook Air M3","ref":"macbook_air_m3.html"},{"type":"extras","title":"Elixir YOLO - Benchmarks on MacBook Air M3","doc":"EPS: `coreml` \n\n**Ultralytics YOLO v11 nano** - `yolo11n`\n\n```\n> mix run benchmarks/ultralytics_yolo.exs coreml models/yolo11n.onnx\n\nOperating System: macOS\nCPU Information: Apple M3\nNumber of Available Cores: 8\nAvailable memory: 16 GB\nElixir 1.18.3\nErlang 27.3.3\nJIT enabled: true\n```\n\n| Name | ips | average  | median | 99th %\n| ---  | --- | ---      | ---    |\n| preprocess | 975.38 | 1.03 ms | 0.99 ms | 1.38 ms\n| postprocess | 276.86 | 3.61 ms |3.56 ms | 3.98 ms\n| run         | 64.07 | 15.61 ms | 15.63 ms | 17.66 ms\n\nAround 20ms from the frame to the detections, which means ~50 FPS, not bad!\n\n**Ultralytics YOLO v11 medium** - `yolo11m`\n\n```terminal\n> mix run benchmarks/ultralytics_yolo.exs coreml models/yolo11m.onnx\n\nOperating System: macOS\nCPU Information: Apple M3\nNumber of Available Cores: 8\nAvailable memory: 16 GB\nElixir 1.18.3\nErlang 27.3.3\nJIT enabled: true\n```\n\n| Name | ips | average  | median | 99th %\n| ---  | --- | ---      | ---    |\n| preprocess | 892.35 | 1.12 ms | 1.09 ms | 1.67 ms\n| postprocess | 265.45 | 3.77 ms | 3.70 ms | 4.36 ms\n| run | 25.20  | 39.68 ms  | 39.68 ms       43.92 ms\n\n\nAround 22FPS.","ref":"macbook_air_m3.html#elixir-yolo"},{"type":"extras","title":"Python Ultralytics - Benchmarks on MacBook Air M3","doc":"Let's see how performs, on the same machine, what I consider the benchmark - the official Ultralytics library:\n\n**Ultralytics YOLO v11 nano** - `yolo11n`\n\n```terminal\n> python benchmarks/ultralytics_yolo.py models/yolo11n.pt cpu\n\n0: 384x640 17 persons, 4 bicycles, 7 cars, 1 truck, 1 traffic light, 25.8ms\nSpeed: 1.2ms preprocess, 25.8ms inference, 0.5ms postprocess per image at shape (1, 3, 384, 640)\n```\n\nPre and post-processing are faster (not by much) and inference is definitely slower than the elixir version, probably due to Ortex coreml acceleration.\n\nWhen running the benchmark with `mps` acceleration (if you have a Mac with Apple Silicon), inference is much faster (even than Elixir YOLO with ONNX model), but pre and post-processing are slower.\n\n```\n> python benchmarks/ultralytics_yolo.py models/yolo11n.pt mps\n\n0: 384x640 17 persons, 4 bicycles, 7 cars, 1 truck, 1 traffic light, 9.2ms\nSpeed: 1.4ms preprocess, 9.2ms inference, 5.3ms postprocess per image at shape (1, 3, 384, 640)\n```\n\n**Ultralytics YOLO v11 medium** - `yolo11m`\n\n\n```terminal\n> python benchmarks/ultralytics_yolo.py models/yolo11m.pt mps\n\n0: 384x640 16 persons, 7 bicycles, 10 cars, 1 truck, 4 traffic lights, 3 backpacks, 1 handbag, 23.3ms\nSpeed: 1.5ms preprocess, 23.3ms inference, 11.8ms postprocess per image at shape (1, 3, 384, 640)\n```","ref":"macbook_air_m3.html#python-ultralytics"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.0"}}